<?php
/**
 * Module Name: SEOSlides Core
 * Activation:  hidden
 * Description: Core module for SEOSlides functionality
 * Version:     0.1
 * Author:      10up
 */

// Object requirements
require_once 'objects/SEOSlides_Slideset.php';
require_once 'objects/SEOSlides_Slide.php';
require_once 'objects/SEOSlides_CanvasObject.php';
if ( ! class_exists( 'Catalyst_API' ) ) {
	require_once 'objects/class-catalyst-api.php';
}

/**
 * Core Functionality of the SEOSlides plugin.
 *
 * @package SEOSlides
 * @subpackage SEOSlides_Core
 * @since 0.1
 */
class SEOSlides_Core {
	/**
	 * @var int Time the class was instantiated. Use for busting certain URL caches.
	 */
	protected $_cache_bust;

	/**
	 * Standard constructor. Wires up actions and filters.
	 */
	public function __construct() {
		$this->install();
		$this->_cache_bust = time();

		// Wire actions
		add_action( 'init',                                          array( $this, 'custom_image_sizes' ) );
		add_action( 'init',                                          array( $this, 'custom_rewrites' ) );
		add_action( 'seoslides_register_cpts',                       array( $this, 'custom_rewrites' ), 11 );
		add_action( 'seoslides_register_cpts',                       array( $this, 'register_cpts' ) );
		add_action( 'admin_enqueue_scripts',                         array( $this, 'admin_enqueue_scripts' ) );
		add_action( 'wp_enqueue_scripts',                            array( $this, 'slideset_enqueue_scripts' ) );
		add_action( 'wp_print_styles',                               array( $this, 'remove_theme_styles' ) );
		add_action( 'right_now_content_table_end',                   array( $this, 'display_presentation_count' ) );
		add_action( 'manage_seoslides-slideset_posts_custom_column', array( $this, 'filter_custom_columns' ), 10, 2 );
		add_action( 'wp_headers',                                    array( $this, 'redirect_to_first' ), 10, 2 );
		add_action( 'admin_bar_menu',                                array( $this, 'presentation_toolbar' ) );
		add_action( 'admin_menu',                                    array( $this, 'add_menu' ), 100 );
		add_action( 'admin_head-post-new.php',                       array( $this, 'presentation_help_tabs' ) );
		add_action( 'admin_head-post.php',                           array( $this, 'presentation_help_tabs' ) );
		add_action( 'admin_head-edit.php',                           array( $this, 'hide_view_switcher' ), 100 );

		// Wire filters
		add_filter( 'manage_seoslides-slideset_posts_columns',         array( $this, 'filter_list_table_columns' ) );
		add_filter( 'manage_edit-seoslides-slideset_sortable_columns', array( $this, 'filter_list_sortable_columns' ) );
		add_filter( 'post_row_actions',                                array( $this, 'remove_quick_edit' ), 10, 2 );
		add_filter( 'single_template',                                 array( $this, 'template_override' ) );
		add_filter( 'default_content',                                 array( $this, 'default_content' ), 10, 2 );
		add_filter( 'post_updated_messages',                           array( $this, 'presentation_messages' ) );
		add_filter( 'catalyst_api_key_name',                           array( $this, 'catalyst_key' ) );
		add_filter( 'default_hidden_meta_boxes',                       array( $this, 'custom_hidden_meta' ), 10, 2 );
		add_filter( 'post_row_actions',                                array( $this, 'post_row_actions' ), 10, 2 );
		add_filter( 'clean_url',                                       array( $this, 'disable_rocketloader' ) );
		add_filter( 'post_type_link',                                  array( $this, 'slide_permalink' ), 10, 2 );
	}

	/**
	 * If the plugin is not installed, install it and set up appropriate DB values.
	 */
	public function install() {
		$installed = get_option( 'seoslides_version' );

		switch( $installed ) {
			case false:
				// Plugin not previously installed.
				add_option( 'seoslides_version', SEOSLIDES_VERSION, '', 'no' );
				add_option( 'seoslides_logo', 'default', '', 'no' );
				add_option( 'seoslides_logo_url', 'https://seoslides.com', '', 'no' );
				add_option( 'seoslides_logo_title', 'seoslides', '', 'no' );
				break;
		}
	}

	/**
	 * Get the API key for the Catalyst ecommerce gateway.
	 *
	 * @param $api_index
	 *
	 * @return string
	 */
	public function catalyst_key( $api_index ) {
		return 'seoslides_api_key';
	}

	/**
	 * Shortcut to register all CPTs
	 *
	 * @return void
	 */
	public function register_cpts() {
		$this->register_presentations();
		$this->register_slides();
	}

	/**
	 * Register the presentation post type.
	 */
	public function register_presentations() {
		$labels = array(
			'name'               => __( 'Presentations', 'seoslides_translate' ),
			'singular_name'      => __( 'Presentation', 'seoslides_translate' ),
			'add_new'            => __( 'Add New', 'seoslides_translate' ),
			'add_new_item'       => __( 'Add New Presentation', 'seoslides_translate' ),
			'edit_item'          => __( 'Edit Presentation', 'seoslides_translate' ),
			'new_item'           => __( 'New Presentation', 'seoslides_translate' ),
			'all_items'          => __( 'All Presentations', 'seoslides_translate' ),
			'view_item'          => __( 'View Presentation', 'seoslides_translate' ),
			'search_items'       => __( 'Search Presentations', 'seoslides_translate' ),
			'not_found'          => __( 'No presentations found', 'seoslides_translate' ),
			'not_found_in_trash' => __( 'No presentations found in Trash', 'seoslides_translate' ),
			'parent_item_colon'  => '',
			'menu_name'          => __( 'Presentations', 'seoslides_translate' )
		);

		$args = array(
			'labels'             => $labels,
			'public'             => true,
			'publicly_queryable' => true,
			'show_ui'            => true,
			'show_in_menu'       => true,
			'query_var'          => true,
			'rewrite'            => array( 'slug' => 'slides', 'with_front' => false ),
			'capability_type'    => 'post',
			'has_archive'        => false,
			'hierarchical'       => false,
			'menu_position'      => 20,
			'supports'           => array( 'title', 'author' )
		);

		register_post_type( 'seoslides-slideset', $args );
	}

	/**
	 * Register the slide post type.
	 */
	public function register_slides() {
		$labels = array(
			'name'               => __( 'Presentation Slides', 'seoslides_translate' ),
			'singular_name'      => __( 'Presentation Slide', 'seoslides_translate' ),
			'add_new'            => __( 'Add New', 'seoslides_translate' ),
			'add_new_item'       => __( 'Add New Slide', 'seoslides_translate' ),
			'edit_item'          => __( 'Edit Slide', 'seoslides_translate' ),
			'new_item'           => __( 'New Slide', 'seoslides_translate' ),
			'all_items'          => __( 'All Slides', 'seoslides_translate' ),
			'view_item'          => __( 'View Slide', 'seoslides_translate' ),
			'search_items'       => __( 'Search Slides', 'seoslides_translate' ),
			'not_found'          => __( 'No slides found', 'seoslides_translate' ),
			'not_found_in_trash' => __( 'No slides found in Trash', 'seoslides_translate' ),
			'parent_item_colon'  => ''
		);

		$args = array(
			'labels'             => $labels,
			'public'             => false,
			'publicly_queryable' => false,
			'show_ui'            => false,
			'show_in_menu'       => false,
			'query_var'          => false,
			'capability_type'    => 'post',
			'has_archive'        => false,
			'hierarchical'       => false,
			'menu_position'      => null,
			'rewrite'            => false
		);

		register_post_type( 'seoslides-slide', $args );
	}

	/**
	 * Add custom rewrite rules to make sure individual slide URLs direct to the appropriate slideset.
	 */
	public function custom_rewrites() {
		add_rewrite_tag( '%seoslides-slide%', '(.+)' );
		add_rewrite_tag( '%seoslides-first%', '([0-9])' );
		add_rewrite_rule( '^slides/?$', 'index.php?seoslides-first=1', 'top' );
		add_rewrite_rule( '^slides/(.+)/(.+)/?$', 'index.php?seoslides-slideset=$matches[1]&seoslides-slide=$matches[2]', 'top' );
	}

	/**
	 * If the seoslides-first query arg is set, redirect to the most recent published presentation.
	 *
	 * @param array $headers
	 * @param WP    $wp
	 *
	 * @return array
	 */
	public function redirect_to_first( $headers, $wp ) {
		if ( ! isset( $wp->query_vars['seoslides-first'] ) || 1 !== (int) $wp->query_vars['seoslides-first'] ) {
			return;
		}

		// Get the most resent publication
		$sets = get_posts(
			array(
				'post_type'   => 'seoslides-slideset',
			    'post_status' => 'publish',
			    'numberposts' => 1,
			)
		);

		if ( 0 === count( $sets ) ) {
			return;
		}

		$slideset = new SEOSlides_Slideset( $sets[0] );

		header( 'Location: '.  trailingslashit( trailingslashit( get_permalink( $slideset->ID ) ) . $slideset->first_slide()->slug ), true, 307 );

		SEOSlides_Core::force_exit();
	}

	/**
	 * Force a PHP exit. Useful for wiring to action hooks.
	 */
	public static function force_exit() {
		exit();
	}

	/**
	 * Build out the strings needed for JS to display properly in a local language.
	 *
	 * @uses wp_localize_script()
	 *
	 * @param string $handle Script we're localizing
	 */
	protected function script_translations( $handle ) {
		$strings = array(
			'label_no_bg'         => __( 'None', 'seoslides_translate' ),
			'label_edit'          => __( 'Edit', 'seoslides_translate' ),
			'label_edit_slide'    => __( 'Edit this slide', 'seoslides_translate' ),
			'label_delete'        => __( 'Delete', 'seoslides_translate' ),
			'label_delete_slide'  => __( 'Delete this slide', 'seoslides_translate' ),
			'label_trash'         => __( 'Trash', 'seoslides_translate' ),
			'label_trash_slide'   => __( 'Move slide to trash', 'seoslides_translate' ),
			'label_restore'       => __( 'Restore', 'seoslides_translate' ),
			'label_restore_slide' => __( 'Restore slide from trash', 'seoslides_translate' ),
			'message_show_trash'  => __( 'Show Trashed Slides', 'seoslides_translate' ),
			'label_master'        => __( 'Slide Master', 'seoslides_translate' ),
			'save_master'         => __( 'Save Slide Master', 'seoslides_translate' ),
			'label_overview'      => __( 'Default Slide Layout', 'seoslides_translate' ),
			'label_defaults'      => __( 'Text Editor Defaults', 'seoslides_translate' ),
			'layout_image'        => __( 'Double-click to Insert Image', 'seoslides_translate' ),
			'layout_text'         => __( 'Click to Insert Text', 'seoslides_translate' ),
			'layout_headline'     => __( 'Click to Insert Headline', 'seoslides_translate' ),
			'layout_subheading'   => __( 'Click to Insert Subtitle', 'seoslides_translate' ),
			'edit_slide'          => __( 'Edit Slide', 'seoslides_translate' ),
			'save_slide'          => __( 'Save Slide', 'seoslides_translate' ),
			'slide_notes'         => __( 'Slide Notes', 'seoslides_translate' ),
			'layout_title'        => __( 'Title and Subtitle Layout', 'seoslides_translate' ),
			'layout_standard'     => __( 'Title and Text Layout', 'seoslides_translate' ),
			'layout_textonly'     => __( 'Text Layout', 'seoslides_translate' ),
			'layout_imageonly'    => __( 'Image Layout', 'seoslides_translate' ),
			'layout_rightimage'   => __( 'Text with Image on Right Layout', 'seoslides_translate' ),
			'layout_leftimage'    => __( 'Text with Image on Left Layout', 'seoslides_translate' ),
			'remove_media'        => __( 'Remove Media', 'seoslides_translate' ),
			'use_media'           => __( 'Use as Background Media', 'seoslides_translate' ),
			'set_media'           => __( 'Set Background Media', 'seoslides_translate' ),
			'wysiwyg_textarea'    => __( 'Textarea', 'seoslides_translate' ),
			'wysiwyg_menu'        => __( 'Insert Text', 'seoslides_translate' ),
			'wysiwyg_icon'        => SEOSLIDES_URL . '/img/text-icon.png',
			'image_name'          => __( 'Image', 'seoslides_translate' ),
			'image_menu'          => __( 'Insert Image', 'seoslides_translate' ),
			'image_icon'          => SEOSLIDES_URL . '/img/image-icon.png',
			'image_select'        => __( 'Select Image', 'seoslides_translate' ),
			'close'               => __( 'Close', 'seoslides_translate' ),
			'seo_title'           => __( 'Title:', 'seoslides_translate' ),
			'seo_description'     => __( 'Description:', 'seoslides_translate' ),
			'seo_keywords'        => __( 'Keywords:', 'seoslides_translate' ),
			'background'          => __( 'Background Settings', 'seoslides_translate' ),
			'hex_value'           => __( 'Hex Value', 'seoslides_translate' ),
			'choose_media'        => __( 'Embed Image/Video', 'seoslides_translate' ),
			'not_image'           => __( "Sorry, that file doesn't appear to be an image.", 'seoslides_translate' ),
			'not_image_or_video'  => __( "Sorry, that file doesn't appear to be an image or a video.", 'seoslides_translate' ),
			'video_only'          => __( ' Video', 'seoslides_translate' ),
			'right_click'         => __( 'Right-click on the canvas above to add elements to the slide.', 'seoslides_translate' ),
			'no_title'            => __( '(no title)', 'seoslides_translate' ),
			'oembed_video'        => __( 'YouTube/Vimeo Source', 'seoslides_translate' ),
			'oembed_valid'        => sprintf( '%s - <span class="valid">%s</span>', __( 'YouTube/Vimeo Source', 'seoslides_translate' ), __( 'Format Valid', 'seoslides_translate' ) ),
			'oembed_invalid'      => sprintf( '%s - <span class="invalid">%s</span>', __( 'YouTube/Vimeo Source', 'seoslides_translate' ), __( 'Invalid URL', 'seoslides_translate' ) ),
			'oembed_helper'       => __( 'Paste the URL to a YouTube or Vimeo video.', 'seoslides_translate' ),
			'label_font'          => __( 'Default Body Font', 'seoslides_translate' ),
			'label_font_size'     => __( 'Default Body Font Size', 'seoslides_translate' ),
			'label_font_color'    => __( 'Default Body Font Color', 'seoslides_translate' ),
			'label_h1_font'       => __( 'Default Header Font', 'seoslides_translate' ),
			'label_h1_font_size'  => __( 'Default Header Font Size', 'seoslides_translate' ),
			'label_h1_font_color' => __( 'Default Header Font Color', 'seoslides_translate' ),
			'insert_link'         => __( 'Save Backlink', 'seoslides_translate' ),
			'link_title'          => __( 'Insert/Edit Embed Backlink', 'seoslides_translate' ),
		);

		wp_localize_script( $handle, 'seoslides_i18n', $strings );
	}

	/**
	 * Enqueue the mediaelement JS library, with a fallback in case we're not using WordPress 3.6.
	 */
	protected function enqueue_mediaelement() {
		if ( 1 === version_compare( '3.6', get_bloginfo( 'version' ) ) ) {
			wp_register_script( 'mediaelement', SEOSLIDES_URL . 'js/lib/mediaelement/mediaelement-and-player.js', array( 'jquery' ), '2.11.3', 1 );
			wp_register_script( 'wp-mediaelement', SEOSLIDES_URL . 'js/lib/mediaelement/wp-mediaelement.js', array( 'mediaelement' ), false, 1 );

			wp_register_style( 'mediaelement', SEOSLIDES_URL . 'js/lib/mediaelement/mediaelementplayer.css' );
			wp_register_style( 'wp-mediaelement', SEOSLIDES_URL . 'js/lib/mediaelement/wp-mediaelement.css', array( 'mediaelement' ) );
		}
		wp_enqueue_script( 'wp-mediaelement' );
		wp_enqueue_style( 'wp-mediaelement' );
	}

	/**
	 * Enqueue a plugin script file.
	 *
	 * @param string $handle
	 * @param array  $deps
	 * @param bool   $footer
	 */
	protected function enqueue_script( $handle, $deps = array(), $footer = false ) {
		if ( defined( 'SCRIPT_DEBUG' ) && true === SCRIPT_DEBUG ) {
			wp_enqueue_script( $handle, SEOSLIDES_URL . "js/{$handle}.src.js", $deps, SEOSLIDES_VERSION . $this->_cache_bust, $footer );
		} else {
			wp_enqueue_script( $handle, SEOSLIDES_URL . "js/{$handle}.min.js", $deps, SEOSLIDES_VERSION, $footer );
		}
	}

	/**
	 * Enqueue scripts and styles used on the admin side of the plugin.
	 */
	public function admin_enqueue_scripts() {
		$current_screen = get_current_screen();

		if ( 'seoslides-slideset' === $current_screen->post_type ) {
			wp_register_style( 'dashicons', SEOSLIDES_URL . 'css/dashicons.css', array(), SEOSLIDES_VERSION );
			wp_enqueue_style( 'dashicons' );

			if ( 'post' === $current_screen->base ) {
				$admin_deps = array( 'utils', 'jquery', 'wp-mediaelement', 'wpdialogs', );

				wp_enqueue_script( 'wp-color-picker' );
				wp_enqueue_style( 'wp-color-picker' );
				wp_enqueue_style( 'wp-jquery-ui-dialog' );

				// Thickbox for selecting background images
				wp_enqueue_script( 'media-upload' );
				wp_enqueue_style( 'thickbox' );

				$this->enqueue_mediaelement();

				$this->enqueue_script( 'poly_fill', null );
				$this->enqueue_script( 'seoslides_admin', $admin_deps, true );

				$slideset = new SEOSlides_Slideset( get_the_ID() );

				$js_variables = array(
					'ajaxurl'          => admin_url( 'admin-ajax.php' ),
					'slideset'         => get_the_ID(),
					'slideset_data'    => $slideset,
					'slide_default'    => $slideset->default_slide(),
					'create_nonce'     => wp_create_nonce( 'seoslides_create' ),
					'update_nonce'     => wp_create_nonce( 'seoslides_update' ),
					'delete_nonce'     => wp_create_nonce( 'seoslides_delete' ),
					'trash_nonce'      => wp_create_nonce( 'seoslides_trash' ),
					'restore_nonce'    => wp_create_nonce( 'seoslides_restore' ),
					'menu_order_nonce' => wp_create_nonce( 'seoslides_order' ),
					'thickbox_noimage' => esc_url( admin_url( 'images/no.png' ) ),
					'thickbox_spinner' => esc_url( admin_url( 'images/wpspin_light.gif' ) ),
					'thickbox_yes'     => esc_url( admin_url( 'images/yes.png' ) ),
				);

				wp_localize_script( 'seoslides_admin', 'seoslides', $js_variables );
				wp_localize_script( 'seoslides_admin', 'CKEDITOR_BASEPATH', SEOSLIDES_URL . 'js/lib/ckeditor/' );
				$this->script_translations( 'seoslides_admin' );

				wp_enqueue_script( 'ckeditor', SEOSLIDES_URL . 'js/lib/ckeditor/ckeditor.js', null, SEOSLIDES_VERSION, true );
			} else {
				$this->enqueue_script( 'seoslides_list', array( 'jquery' ), true );
				$this->script_translations( 'seoslides_list' );
			}

			wp_enqueue_style( 'seoslides-admin-slides', SEOSLIDES_URL . '/css/admin-styles-slides.css', array(), SEOSLIDES_VERSION, 'all' );
		}
		wp_enqueue_style( 'seoslides-bucket', SEOSLIDES_URL . '/js/lib/canvas/css/bucket.css', array(), SEOSLIDES_VERSION, 'all' );
		wp_enqueue_style( 'seoslides-admin', SEOSLIDES_URL . '/css/admin-styles.css', array(), SEOSLIDES_VERSION, 'all' );
	}

	/**
	 * Enqueue scripts and styles only for the presentations.
	 */
	public function slideset_enqueue_scripts() {
		if ( ! is_singular( 'seoslides-slideset' ) ) {
			return;
		}

		wp_register_script( 'modernizr', SEOSLIDES_URL . 'js/lib/modernizr.js', array(), '2.6.2' );

		// Styles
		wp_enqueue_style( 'gfont-lato', 'http://fonts.googleapis.com/css?family=Lato:400,700,400italic' );
		wp_enqueue_style( 'seoslides-front', SEOSLIDES_URL . 'css/front-end.css', array( 'gfont-lato' ), SEOSLIDES_VERSION, 'screen' );
		wp_enqueue_style( 'seoslides-print', SEOSLIDES_URL . 'css/print.css', array(), SEOSLIDES_VERSION, 'print' );
		wp_register_style( 'dashicons', SEOSLIDES_URL . 'css/dashicons.css', array(), SEOSLIDES_VERSION );
		wp_enqueue_style( 'dashicons' );

		// Video
		$this->enqueue_mediaelement();

		// Scripts
		$this->enqueue_script( 'seoslides_front', array( 'jquery', 'modernizr', 'wp-mediaelement' ), true );
		$this->script_translations( 'seoslides_front' );

		$embedID = '';
		if ( get_query_var( 'seoslides-embed' ) ) {
			$slide   = get_query_var( 'seoslides-embed-slide' ) ? get_query_var( 'seoslides-embed-slide' ) : '';
			$embedID = SEOSlides_Module_Provider::get( 'SEOSlides Embed' )->get_embed_unique_id( get_the_ID(), $slide );
		}

		wp_localize_script( 'seoslides_front', 'seoslides', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ), 'slideset' => get_the_ID(), 'embedID' => $embedID ) );
	}

	/**
	 * Forcably remove theme styles.
	 */
	public function remove_theme_styles() {
		if ( ! is_singular( 'seoslides-slideset' ) ) {
			return;
		}

		global $wp_styles;

		$theme_ss_uri = get_stylesheet_directory_uri();

		foreach( $wp_styles->queue as $queued ) {
			$sheet = $wp_styles->registered[ $queued ];

			if ( false !== strpos( $sheet->src, $theme_ss_uri ) ) {
				$wp_styles->dequeue( $queued );
			}
		}
	}

	/**
	 * Add the number of presentations in the system to the Dashboard's Right Now widget.
	 */
	public function display_presentation_count() {
		$num_pres = wp_count_posts( 'seoslides-slideset' );

		$num = number_format_i18n( $num_pres->publish );
		$text = _n( 'Presentation', 'Presentations', intval( $num_pres->publish ), 'seoslides_translate' );

		if (  current_user_can( 'edit_posts' ) ) {
			$num = "<a href='edit.php?post_type=seoslides-slideset'>$num</a>";
			$text = "<a href='edit.php?post_type=seoslides-slideset'>$text</a>";
		}

		echo '<tr><td class="first b b">' . $num . '</td>';
		echo '<td class="t">' . $text . '</td></tr>';
	}

	/**
	 * Set up the columns for the Presentation list page.
	 *
	 * @param array $post_columns
	 *
	 * @return array
	 */
	public function filter_list_table_columns( $post_columns ) {
		$columns = array(
			'cb'           => '<input type="checkbox" />',
			'thumbnail'    => __( 'Preview', 'seoslides_translate' ),
			'title'        => __( 'Title', 'seoslides_translate' ),
			'slide_count'  => __( 'Slides', 'seoslides_translate' ),
			'author'       => __( 'Author', 'seoslides_translate' ),
			'date'         => __( 'Date', 'seoslides_translate' ),
			'updated'      => __( 'Updated', 'seoslides_translate' )
		);

		return $columns;
	}

	/**
	 * Filter the display of the custom Presentation columns.
	 *
	 * @param string $column
	 * @param int    $slideset_id
	 *
	 * @return void
	 */
	public function filter_custom_columns( $column, $slideset_id ) {
		switch( $column ) {
			case 'thumbnail':
				$slideset = new SEOSlides_Slideset( $slideset_id );
				$title = $slideset->title;

				$slides = array_filter( $slideset->slides, array( 'SEOSlides_Slide', 'slide_is_published' ) );

				if ( count( $slides ) > 0 ) {
					$cover = $slideset->first_slide();

					if ( current_user_can( 'edit_post', $slideset_id ) )
						echo '<a href="' . get_edit_post_link( $slideset_id ) . '">';

					echo '<div class="list thumbnail">';

					if ( false === $cover->preview || 'false' === $cover->preview ) {
						// There is no preview thumbnail for this image, so we'll try to create one instead
						echo "<section class='slide'><div class='slide-body' {$cover->style}>";

						if ( ! empty( $cover->oembed ) ) {
							$embed_url = SEOSlides_Slide::get_embed_url( $cover->oembed );
							$thumb_url = SEOSlides_Slide::get_embed_thumbnail( $cover->oembed );

							if ( ! is_wp_error( $embed_url ) && ! is_wp_error( $thumb_url )) {
								echo '<img class="seoslides_iframe_thumb" src="' . $thumb_url . '" />';
								echo '<iframe class="seoslides_iframe" src="' . $embed_url . '" frameborder="0" webkitAllowFullScreen mozallowfullscreen allowFullScreen></iframe>';
							}
						}

						foreach( $cover->objects as $object ) {
							echo '<div ';
							echo "data-element='{$object->element_id}' ";
							echo "data-plugin='{$object->plugin_id}' ";
							echo "data-width='{$object->settings->size->w}' ";
							echo "data-height='{$object->settings->size->h}' ";
							echo "data-top='{$object->settings->position->top}' ";
							echo "data-left='{$object->settings->position->left}' ";

							$style = 'position:absolute; ';
							$style .= 'top:' . $object->settings->position->top . 'px; ';
							$style .= 'left:' . $object->settings->position->left . 'px; ';
							$style .= 'width:' . $object->settings->size->w . 'px; ';
							$style .= 'height:' . $object->settings->size->h . 'px;';

							echo "style='{$style}'";

							echo '>';
							if ( isset( $object->settings->content ) ) {
								echo $object->settings->content;
							}
							echo '</div>';
						}

						echo '</div></section>';
					} else {
						echo "<img src=\"{$cover->preview}\" title=\"{$cover->title}\" />";
					}

					echo '</div>';

					if ( current_user_can( 'edit_post', $slideset_id ) )
						echo '</a>';
				}

				break;
			case 'description':
				$slideset = $this->get_slideset( $slideset_id );

				echo $slideset->excerpt;
				break;
			case 'slide_count':
				echo self::get_slide_count( $slideset_id );
				break;
			case 'updated':
				$slideset = $this->get_slideset( $slideset_id );
				$updated = mysql2date( 'G', $slideset->last_updated_gmt, false );
				$diff = time() - $updated;

				if ( $diff > 0 && $diff < DAY_IN_SECONDS ) {
					$time = sprintf( __( '%s ago' ), human_time_diff( $updated ) );
				} else {
					$time = mysql2date( __( 'Y/m/d' ), $slideset->last_updated );
				}

				echo $time;
				break;
		}
	}

	/**
	 * Make the Updated column to be sortable
	 *
 	* @param array $columns
	 *
 	* @return array
 	*/
	public function filter_list_sortable_columns( $columns ) {
		$columns['updated'] = 'modified';
		return $columns;
	}

	/**
	 * Remove the Quick Edit option from the list table.
	 *
	 * @param array  $actions
	 * @param object $post
	 *
	 * @return array
	 */
	public function remove_quick_edit( $actions, $post ) {
		if ( 'seoslides-slideset' == $post->post_type ) {
			unset( $actions['inline hide-if-no-js'] );
		}

		return $actions;
	}

	/**
	 * Intercept the template redirect request and inject the one bundled by the plugin if it's not set by the theme.
	 *
	 * If there is no global $post, or if the post type for the global $post is not 'seoslides-slideset,' skip any changes.
	 * Also, if the theme defines a template for 'seoslides-slideset,' skip changes and use that template instead.
	 *
	 * @param string $template Template file.
	 *
	 * @return string
	 */
	public function template_override( $template ) {
		global $post;

		if ( null == $post || 'seoslides-slideset' != $post->post_type || strpos( $template, 'seoslides-slideset' ) > 0 ) {
			return $template;
		}

		return SEOSLIDES_PATH . 'templates/single-seoslides-slideset.php';
	}

	/**
	 * Populate a static template with actual slide parameters.
	 *
	 * @param string $template  Template to populate.
	 * @param array  $slideargs Associative array of parameters to use for population.
	 *
	 * @return string
	 */
	public static function populate_template( $template, $slideargs ) {
		foreach( $slideargs as $key => $value ) {
			switch( $key ) {
				case 'image':
					$template = str_replace( "<%= $key %>", "<div class='seoslide_image'>{$value}</div>", $template );
					break;
				case 'content':
					$template = str_replace( "<%= $key %>", "<div class='seoslide_content'>{$value}</div>", $template );
					break;
				default:
					$template = str_replace( "<%= $key %>", $value, $template );
			}
		}

		return $template;
	}

	/**
	 * Get the total number of slides assigned to a particular presentation.
	 *
	 * @param int $slideset_id
	 *
	 * @return int
	 */
	public static function get_slide_count( $slideset_id ) {
		$cache_key = 'slideset_' . $slideset_id;

		$count = wp_cache_get( $cache_key, 'counts' );

		if ( false !== $count ) {
			return $count;
		}

		$slideset = new SEOSlides_Slideset( $slideset_id );

		$slides = array_filter( $slideset->slides, array( 'SEOSlides_Slide', 'slide_is_published' ) );

		$count = count( $slides );

		wp_cache_set( $cache_key, $count, 'counts' );

		return $count;
	}

	/**
	 * Register custom image sizes so appropriate thumbnails are available when needed.
	 *
	 * @uses add_image_size()
	 */
	public function custom_image_sizes() {
		add_image_size( 'seoslides-bg-thumb', 160, 90 );
	}

	/**
	 * Filter the default content of a new post.
	 *
	 * @param string  $post_content
	 * @param WP_Post $post
	 */
	public function default_content( $post_content, $post ) {
		if ( 'seoslides-slideset' === $post->post_type ) {
			$default_css = <<<EOD
section.slide {}
.deck-status {}
EOD;

			$default_css = apply_filters( 'seoslides_default_css', $default_css );

			add_post_meta( $post->ID, '_seoslides_custom_css', $default_css, true );
		}

		return $post_content;
	}

	/**
	 * Fetch a presentation from the database.
	 *
	 * @param int $slideset_id
	 *
	 * @return null|SEOSlides_Slideset
	 */
	public function get_slideset( $slideset_id ) {
		// Sanitize slideset ID
		$slideset_id = (int) $slideset_id ;

		$slideset = new SEOSlides_Slideset( $slideset_id );

		if ( 0 === $slideset->ID ) {
			return null;
		}

		return $slideset;
	}

	/**
	 * Get a specific slide from the database.
	 *
	 * @param int $slide_id
	 *
	 * @return object|null
	 */
	public function get_slide( $slide_id ) {
		// Sanitize slide id
		$slide_id = (int) $slide_id;

		$slide = new SEOSlides_Slide( $slide_id );

		if ( 0 === $slide->ID ) {
			return null;
		}

		return $slide;
	}

	/**
	 * Add messages specific to slideset save/error actions.
	 *
	 * @param array $messages
	 *
	 * @global WP_Post $post
	 * @global int     $post_ID
	 *
	 * @return array
	 */
	public function presentation_messages( $messages = array() ) {
		global $post, $post_ID;

		$use_in_post = ' ' . sprintf( __( 'Use presentatation <a data-nonce="%s" data-presentation="%s" id="%s" href="%s">in a new post</a>.', 'seoslides_translate' ), wp_create_nonce( 'use_in_post' ), $post_ID, 'use_in_post', esc_url( get_permalink( $post_ID ) . '#use_in_post' ) );

		$messages['seoslides-slideset'] = array(
			0  => '', // Unused. Messages start at index 1.
			1  => sprintf( __( 'Presentation updated. <a href="%s">View presentation</a>.', 'seoslides_translate' ), esc_url( get_permalink( $post_ID ) ) ) . $use_in_post,
			2  => __( 'Custom field updated.' ),
			3  => __( 'Custom field deleted.' ),
			4  => __( 'Presentation updated.', 'seoslides_translate' ),
			/* translators: %s: date and time of the revision */
			5  => isset( $_GET['revision'] ) ? sprintf( __( 'Presentation restored to revision from %s', 'seoslides_translate' ), wp_post_revision_title( (int) $_GET['revision'], false ) ) : false,
			6  => sprintf( __( 'Presentation published. <a href="%s">View presentation</a>', 'seoslides_translate' ), esc_url( get_permalink( $post_ID ) ) ) . $use_in_post,
			7  => __( 'Presentation saved.', 'seoslides_translate' ),
			8  => sprintf( __( 'Presentation submitted.', 'seoslides_translate' ), esc_url( add_query_arg( 'preview', 'true', get_permalink( $post_ID ) ) ) ),
			9  => sprintf( __( 'Presentation scheduled for: <strong>%1$s</strong>.', 'seoslides_translate' ),
				// translators: Publish box date format, see http://php.net/date
				date_i18n( __( 'M j, Y @ G:i' ), strtotime( $post->post_date ) ), esc_url( get_permalink( $post_ID ) ) ) . $use_in_post,
			10 => sprintf( __( 'Presentation draft updated.', 'seoslides_translate' ), esc_url( add_query_arg( 'preview', 'true', get_permalink( $post_ID ) ) ) ),
		);

		return $messages;
	}

	/**
	 * Adds a Presentations link to the front-end Toolbar
	 *
	 * @param WP_Admin_Bar $wp_admin_bar The admin bar global instance.
	 */
	public function presentation_toolbar( $wp_admin_bar ) {
		if ( ! is_admin() ) {
			$wp_admin_bar->add_menu( array(
				'parent' => 'appearance',
				'id'     => 'presentations',
				'title'  => __( 'Presentations' ),
				'href'   => add_query_arg( 'post_type', 'seoslides-slideset', admin_url( 'edit.php' ) ),
			) );
		}
	}

	/**
	 * Add the import menu to the Presentations top-level item.
	 *
	 * @uses add_submenu_page
	 */
	public function add_menu() {
		add_submenu_page(
			'edit.php?post_type=seoslides-slideset',
			__( 'Settings', 'seoslides_translate' ),
			__( 'Settings', 'seoslides_translate' ),
			'edit_posts',
			'settings',
			array( $this, 'menu' )
		);

		if ( $this->get_subscription_level() < 20 ) {
			global $submenu;

			$submenu['edit.php?post_type=seoslides-slideset'][50] = array( __( 'Upgrade', 'seoslides_translate' ), 'manage_options', 'https://seoslides.com/upgrade' );
		}

		add_submenu_page(
			'edit.php?post_type=seoslides-slideset',
			__( 'Support', 'seoslides_translate' ),
			__( 'Support', 'seoslides_translate' ),
			'edit_posts',
			'support',
			array( $this, 'support' )
		);
	}

	/**
	 * Handle the core settings menu
	 */
	public function menu() {
		$this->process_postback();
		$api_key = get_option( 'seoslides_api_key', '' );

		settings_errors( 'seoslides' );

		if ( $this->get_subscription_level() < 20 ) : ?>
			<div class="updated">
				<p><?php _e( 'You can use a free license key for 3 imports and 3 embeds on <a href="http://seoslid.es">seoslid.es</a> (our presentation community).', 'seoslides_translate' ); ?></p>
				<p>
					<?php _e( '<a href="https://seoslides.com/upgrade">Upgrade</a> to the pro version for unlimited imports during the beta and unlimited embeds to <a href="http://seoslid.es">seoslid.es</a>.', 'seoslides_translate' ); ?>
					<?php _e( 'The pro version also includes the ability to replace our logo and link with your own in every one of your presentations and embeds (free backlinks!).', 'seoslides_translate' ); ?>
				</p>
			</div>
		<?php endif; ?>

		<div class="wrap">
			<?php screen_icon(); ?>
			<h2>
				<?php _e( 'seoslides Settings', 'seoslides_translate' ); ?>
			</h2>

			<h3>
				<?php _e( 'General Settings', 'seoslides_translate' ); ?>
			</h3>
			<form id="seoslides_settings" method="post">
				<?php wp_nonce_field( __FILE__, '_seoslides_nonce' ); ?>
				<table class="form-table">
					<tbody>
					<?php do_action( 'seoslides_settings_form_top_rows' ); ?>
					<tr valign="top">
						<th scope="row">
							<label for="api_key"><?php _e( 'License Key', 'seoslides_translate' ); ?></label>
						</th>
						<td>
							<input name="api_key" type="text" id="api_key" value="<?php echo esc_attr( $api_key ); ?>" class="regular-text" />
							<span id="api_key_indicator" class="<?php echo empty( $api_key ) ? 'unset' : 'valid'; ?>"></span>
							<p class="description"><?php _e( "Don't have a license key? You can get one <a href=\"https://seoslides.com/upgrade\">here</a>.", 'seoslides_translate' ); ?></p>
						</td>
					</tr>
					<?php do_action( 'seoslides_settings_form_bottom_rows' ); ?>
					</tbody>
				</table>

				<p class="submit"><input type="submit" name="submit" id="submit" class="button button-primary" value="<?php _e( 'Save Changes', 'seoslides_translate' ); ?>"></p>
			</form>
		</div>
<?php
	}

	/**
	 * Handle the user support menu
	 */
	public function support() {
		$step = empty( $_GET['step'] ) ? 0 : (int) $_GET['step'];
?>
		<?php if ( 1 === $step ) : ?>
			<div class="updated">
				<p>
					<?php _e( 'We\'ve got it! Someone from our support team will be in touch soon.', 'seoslides_translate' ); ?>
				</p>
			</div>
		<?php endif; ?>

		<?php $api_key = get_option( 'seoslides_api_key' ); ?>
		<?php $api_root = apply_filters( 'seoslides_api_root', 'https://seoslides.com' ); ?>
		<div class="wrap">
			<?php screen_icon(); ?>
			<h2>
				<?php _e( 'seoslides Support', 'seoslides_translate' ); ?>
			</h2>

			<form id="support-form" method="post" action="<?php echo $api_root; ?>/wp-admin/admin-post.php">
				<input id="seoslides-api_key" name="seoslides-api_key" type="hidden" value="<?php echo esc_attr( $api_key ); ?>" />
				<input id="seoslides-redirect" name="seoslides-redirect" type="hidden" value="<?php echo esc_attr( admin_url( 'edit.php?post_type=seoslides-slideset&page=support&step=1' ) ); ?>" />
				<input id="action" name="action" type="hidden" value="support-request" />

				<p><?php echo sprintf( __( 'Need help with something?  Please check our <a href="%s">comprehensive list of FAQs</a>.', 'seoslides_translate' ), 'https://seoslides.com/faq/'); ?></p>
				<p><?php _e( 'Still need help? Get in touch with our team by submitting the form below:', 'seoslides_translate' ); ?></p>
				<table class="form-table">
					<tbody>
						<?php do_action( 'seoslides_support_form_top_rows' ); ?>
						<tr valign="top">
							<th scope="row">
								<label for="subject"><?php _e( 'Please enter a subject in the form of a short question:', 'seoslides_translate' ); ?></label>
							</th>
							<td>
								<input id="subject" name="subject" type="text" class="large-text ltr" />
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="message"><?php _e( 'Please describe the problem you\'re having', 'seoslides_translate' ); ?></label>
							</th>
							<td>
								<textarea id="message" name="message" rows="6" cols="45" class="large-text ltr"></textarea>
							</td>
						</tr>

						<?php $current_user = wp_get_current_user(); ?>

						<tr valign="top">
							<th scope="row">
								<label for="name"><?php _e( 'Name', 'seoslides_translate' ); ?></label>
							</th>
							<td>
								<input id="name" name="name" type="text" class="regular-text ltr" value="<?php echo esc_attr( $current_user->display_name ); ?>" />
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="contact"><?php _e( 'Email', 'seoslides_translate' ); ?></label>
							</th>
							<td>
								<input id="contact" name="contact" type="text" class="regular-text ltr" value="<?php echo esc_attr( $current_user->user_email ); ?>" />
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="site_url"><?php _e( 'Site URL', 'seoslides_translate' ); ?></label>
							</th>
							<td>
								<input id="site_url" name="site_url" type="text" class="regular-text ltr" value="<?php echo esc_attr( get_option( 'url' ) ); ?>" />
							</td>
						</tr>
						<?php do_action( 'seoslides_support_form_bottom_rows' ); ?>
					</tbody>
				</table>

				<?php submit_button( __( 'Send Support Request', 'seoslides_translate' ), 'button button-primary' ); ?>

			</form>
		</div>
<?php
	}

	/**
	 * Process data from the posted form
	 */
	protected function process_postback() {
		if ( ! isset( $_POST['submit'] ) ) {
			return;
		}

		if ( ! isset( $_POST['_seoslides_nonce'] ) || ! wp_verify_nonce( $_POST['_seoslides_nonce'], __FILE__ ) ) {
			return;
		}

		if ( isset( $_POST['api_key'] ) ) {
			delete_transient( 'seoslides_level' );

			$old_key = get_option( 'seoslides_api_key' );
			$api_key = sanitize_text_field( $_POST['api_key'] );

			if ( $old_key !== $api_key ) {
				// Now validate the key against the remote API
				$catalyst = Catalyst_API::instance();
				$catalyst->set_api_key( $api_key );
				$valid = $catalyst->validate_key();

				if ( $valid ) {
					add_settings_error( 'seoslides', 'settings_updated', __( 'Settings Saved!', 'seoslides_translate' ), 'updated' );
				} else {
					add_settings_error( 'seoslides', 'invalid_key', __( 'Invalid License Key!', 'seoslides_translate' ), 'error' );
					$api_key = '';
				}

				if ( $old_key !== false ) {
					update_option( 'seoslides_api_key', $api_key );
				} else {
					add_option( 'seoslides_api_key', $api_key, '', 'no' );
				}
			}
		}

		do_action( 'seoslides_settings_postback' );
	}

	/**
	 * Hide the Custom CSS meta box by default.
	 *
	 * @param array     $hidden
	 * @param WP_Screen $screen
	 *
	 * @return array
	 */
	public function custom_hidden_meta( $hidden, $screen ) {
		if ( 'seoslides-slideset' !== $screen->id ) {
			return $hidden;
		}

		$hidden[] = 'seoslides_custom_css';

		return $hidden;
	}

	/**
	 * Get the site's subscription level, either from the cache or from the remote API.
	 *
	 * 0 - unregistered
	 * 10 - free
	 * 20 - premium
	 *
	 * @return int
	 */
	public function get_subscription_level() {
		$level = get_transient( 'seoslides_level' );

		if ( false === $level ) {
			$level = 0;
			$api_key = get_option( 'seoslides_api_key', '' );

			if ( ! empty( $api_key ) ) {
				$catalyst = Catalyst_API::instance();
				$catalyst->set_api_key( $api_key );
				$subscriptions = $catalyst->get_subscription_plans();

				if ( in_array( 'c8c8ee061df0601b7eb9ef90946100eded29e3d1e68b4dbe82e05aabb6042f8a', $subscriptions ) ) {
					$level = 20;
				} elseif ( in_array( 'e41a557640b1b8c2e73b5abe0460d10c56fd6052d30da96511ba80acf9fe0ccf', $subscriptions ) ) {
					$level = 10;
				}
			}

			set_transient( 'seoslides_level', $level, 60 * 60 );
		}

		return (int) $level;
	}

	public function presentation_help_tabs() {
		global $typenow;
		if ( is_admin() && 'seoslides-slideset' == $typenow ) {
			$basics  = '<p>' . __( 'This is the Presentation Editing screen. From here, you can:', 'seoslides_translate' ) . '</p>';
			$basics .= '<ul><li>' . __( 'Edit or create a new presentation', 'seoslides_translate' ) . '</li>';
			$basics .= '<li>'     . __( 'Customize the look and feel with CSS', 'seoslides_translate' ) . '</li>';
			$basics .= '<li>'     . __( 'Publish to seoslid.es', 'seoslides_translate' ) . '</li>';
			$basics .= '<li>'     . __( 'Modify your Presentation Link', 'seoslides_translate' ) . '</li></ul>';
			$basics .= '<p>' . __( 'For a more in-depth look at how to edit your presentations, check out the other tabs in this panel. If you&#8217;re still stuck, we&#8217;re here to help &mdash; <a href="">Contact Support</a>.', 'seoslides_translate' ) . '</p>';

			get_current_screen()->add_help_tab( array(
				'id'      => 'seoslides-basics',
				'title'   => _x( 'Basics', 'general help tab', 'seoslides_translate' ),
				'content' => $basics
			) );

			$editing_help  = '<p>' . __( 'seoslides&#8217; editor enables you to create rich, SEO-friendly slideshow presentations with ease.', 'seoslides_translate' ) . '</p>';
			$editing_help .= '<p>' . __( 'In the slide editor you can:', 'seoslides_translate' ) . '</p>';
			$editing_help .= '<ul><li>' . __( 'Choose layouts', 'seoslides_translate' ) . '</li>';
			$editing_help .= '<li>'     . __( 'Add or edit text', 'seoslides_translate' ) . '</li>';
			$editing_help .= '<li>'     . __( 'Embed rich media such as audio or video', 'seoslides_translate' ) . '</li>';
			$editing_help .= '<li>'     . __( 'Change colors, and more!', 'seoslides_translate' ) . '</li></ul>';
			$editing_help .= '<p>' . __( 'Each presentation has a special &#8220;Slide Master&#8221; at the top of the slide list. This slide serves as a template for all of the other slides you add.', 'seoslides_translate' ) . '</p>';
			$editing_help .= '<p>' . __( 'To edit a specific slide, <strong>click that slide&#8217;s preview image or Edit link</strong> in the slides list.', 'seoslides_translate' ) . '</p>';

			get_current_screen()->add_help_tab( array(
				'id'      => 'seoslides-editing',
				'title'   => __( 'Editing Slides', 'seoslides_translate' ),
				'content' => $editing_help
			) );

			$stylesheet_url = plugin_dir_url( __FILE__ );
			$stylesheet_url = str_replace( 'includes/', 'css/front-end.css', $stylesheet_url );

			$css_help  = '<p>' . __( 'The Custom Presentation CSS meta box provides more advanced users with the ability to set custom CSS styles on a per-presentation basis. Display of the meta box can be toggled on and off via Screen Options.', 'seoslides_translate' ) . '</p>';
			$css_help .= '<p>' . __( 'When a new presentation is created, two empty styles are provided in the CSS editor, <code>section.slide{}</code> &mdash; used to style all slides &mdash; and <code>.deck-status{}</code>.', 'seoslides_translate' ) . '</p>';
			$css_help .= '<p>' . sprintf( __( 'For hints on other styles to target, take a look at the Front-End stylesheet located at <code>%s</code>.', 'seoslides_translate' ), $stylesheet_url ). '</p>';

			get_current_screen()->add_help_tab( array(
				'id'      => 'seoslides-css',
				'title'   => __( 'Customizing CSS', 'seoslides_translate' ),
				'content' => $css_help
			) );

			$multiplier_help  = '<p>' . __( 'Depending on your subscription level, you may have the option to publish your presentations remotely to <a href="http://seoslid.es">seoslide.es</a>. A copy of your presentation will be embedded there and will link back to your site.', 'seoslides_translate' ) . '</p>';
			$multiplier_help .= '<p>' . __( 'seoslides Pro users may remotely publish an <strong>unlimited</strong> number of presentatations, while free users are limited to <strong>just one</strong> remote publication. If you&#8217;ve already exhausted your free allowance, <a href="https://seoslides.com/upgrade/" target="_blank">upgrade to Pro today!</a>', 'seoslides_translate' ) . '</p>';
			$multiplier_help .= '<p>' . __( 'Publishing to the seoslid.es:', 'seoslides_translate' ) . '</p>';
			$multiplier_help .= '<ol><li>' . __( 'First, publish your presentation on your site by <strong>clicking the blue Publish button</strong>', 'seoslides_translate' ) . '</li>';
			$multiplier_help .= '<li>'     . __( 'In the Publish meta box, <strong>click the Edit link, next to &#8220;seoslid.es&#8221;</strong>', 'seoslides_translate' ) . '</li>';
			$multiplier_help .= '<li>'     . __( '<strong>Click the &#8220;Publish to seoslid.es now&#8221; button</strong> to send your presentation to the remote server. That&#8217;s it!', 'seoslides_translate' ) . '</li></ol>';
			$multiplier_help .= '<p>' . __( 'Other Options:', 'seoslides_translate' ) . '</p>';
			$multiplier_help .= '<ul><li>' . __( '<strong>Select the &#8220;Sync updates automatically&#8221; checkbox</strong> to auto-update the remote server whenever you update your presentation', 'seoslides_translate' ) . '</li>';
			$multiplier_help .= '<li>'     . __( '<strong>Click the &#8220;Remove from seoslid.es now&#8221; button</strong> to remove your presentation from the remote server', 'seoslides_translate' ) . '</li>';
			$multiplier_help .= '<li>'     . __( 'If your presentation has been published remotely, un-publishing it on your own site will also remove it from <a href="http://seoslid.es/">seoslid.es</a>', 'seoslides_translate' ) . '</li></ul>';

			get_current_screen()->add_help_tab( array(
				'id'      => 'seoslides-multiplier',
				'title'   => __( 'Using seoslid.es', 'seoslides_translate' ),
				'content' => $multiplier_help
			) );

			$presentation_help  = '<p>' . __( '', 'seoslides_translate' ) . '</p>';

			get_current_screen()->add_help_tab( array(
				'id'      => 'seoslides-presentation',
				'title'   => __( 'Presentation Link', 'seoslides_translate' ),
				'content' => $presentation_help
			) );
		} elseif ( is_admin() && 'post' == $typenow ) {
			$embed_presentations  = '<p>' . __( '', 'seoslides_translate' ) . '</p>';

			get_current_screen()->add_help_tab( array(
				'id'      => 'seoslides-embed',
				'title'   => __( 'Embed Presentations', 'seoslides_embed' ),
				'content' => $embed_presentations
			) );
		}
	}

	/**
	 * Filter out the Preview link from presentation drafts.
	 *
	 * @param array   $actions
	 * @param WP_Post $post
	 *
	 * @return array
	 */
	public function post_row_actions( $actions, $post ) {
		if ( 'seoslides-slideset' === $post->post_type && 'draft' === $post->post_status ) {
			unset( $actions['view'] );
		}

		return $actions;
	}

	/**
	 * Hide the view switcher from the All Presentations list table.
	 */
	public function hide_view_switcher() {
		if ( 'seoslides-slideset' == get_current_screen()->post_type ) :
			?>
			<style type="text/css">
				.view-switch {
					display: none;
				}
			</style>
		<?php
		endif;
	}

	/**
	 * Add CloudFlare tags to the main script as an attribute so RocketLoader ignores it.
	 *
	 * @param string $url URL to filter
	 *
	 * @return string
	 */
	public function disable_rocketloader( $url ) {
		// Build out the front script
		if ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) {
			$front_script = SEOSLIDES_URL . 'js/seoslides_front.src.js?ver=' . SEOSLIDES_VERSION . $this->_cache_bust;
		} else {
			$front_script = SEOSLIDES_URL . 'js/seoslides_front.min.js?ver=' . SEOSLIDES_VERSION;
		}


		if ( $front_script === $url ) {
			$url = "$url' data-cfasync='false";
		}

		return $url;
	}

	/**
	 * Filter the slide permalinks such that they return something useful.
	 *
	 * @since 1.0.3
	 *
	 * @param string  $url  Original permalink
	 * @param WP_Post $post Post object
	 *
	 * @return string
	 */
	public function slide_permalink( $url, $post ) {
		if ( 'seoslides-slide' !== get_post_type( $post ) ) {
			return $url;
		}

		// Build pretty permalink
		$slide = new SEOSlides_Slide( $post );
		$url = trailingslashit( get_permalink( $slide->parent( 'ID' ) ) ) . $slide->slug . '/';

		return $url;
	}
}
